/*
 *  readpng, a simple interface to load PNG to an OpenGL compatible structure.
 *  Copyright (C) 2008  Colomban Wendling <ban@herbesfolles.org>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/*
 * creation date: Tue Sep  9 17:00:00 CEST 2008
 * revision date: Wed Sep 10 19:59:24 CEST 2008
 */


#include <stdio.h>
#include <GL/gl.h>


#ifndef READPNG_H_DEFINED
#define READPNG_H_DEFINED


/* OpenGL texture info */
typedef struct s_RPNGTextureGL RPNGTextureGL;
/** readpng's OpenGL texture structure */
struct s_RPNGTextureGL
{
  GLuint    width;          /**< Texture's width */
  GLuint    height;         /**< Texture's heigth */
  
  GLenum    format;         /**< Texture's type (RVB, RVBA, Luminance,
                             *                   Luminance Alpha) */
  GLint     internalFormat; /**< Componants of a texel (# of BPP) */
  
  GLubyte*  texels;         /**< Texture's texel data */
};

/** Type of function used to free allocated data */
typedef void (*RPNGDataFreeFunc) (void* data);
typedef unsigned char readpng_byte;

typedef struct s_RPNGBuffer RPNGBuffer;
/** readpng's buffer structure */
struct s_RPNGBuffer
{
  readpng_byte*   data;   /**< Pointer to the data */
  long            length; /**< Data length */
  long            offset; /**< Current offset in the data. This is helpful to
                           *   walk through the data */
  
  /* binded to free() by default */
  RPNGDataFreeFunc  data_free_func; /**< Function to free data when
                                     *   readpng_buf_free() is called.
                                     *   By default the binded function is
                                     *   stdlib's free() */
};


/* prototypes */
extern RPNGBuffer*    readpng_buf_new             (RPNGDataFreeFunc free_func);
extern void           readpng_buf_free            (RPNGBuffer* buf);
extern RPNGBuffer*    readpng_buf_new_from_stream (FILE* fp);

extern RPNGTextureGL* readpng_tgl_new             (void);
extern void           readpng_tgl_free            (RPNGTextureGL* tgl);
extern RPNGTextureGL* readpng_tgl_new_from_buffer (RPNGBuffer* buf);
extern RPNGTextureGL* readpng_tgl_new_from_stream (FILE* fp);
extern RPNGTextureGL* readpng_tgl_new_from_file   (const char* filename);

extern GLuint         readpng_load_mipmaps_gl     (const char* filename);
extern GLuint         readpng_load_texture_gl     (const char* filename);


#endif /* guard (READPNG_H_DEFINED) */
