/*
 *  Jomar3D, a lobster-running-to-catch game.
 *  Copyright (C) 2006-2008 Jonathan Michalon <studios.chalmion@no-log.org>.
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "jomar3D.h"
#include "utils.h"


/* return the extension of a filename or NULL -- (C) Colomban Wendling GPL v.3 */
const char *
getext (const char *str)
{
  const char *ext = NULL;

  if ((ext = strrchr (str, '.')) != NULL)
  {
    if (ext[1] == '\0')
      return NULL;
    else
      return ++ext;
  }

  return NULL;
}

/* cut the extension of a string */
char *
rmext (char *const str)
{
  const char* ext;
  size_t len;
  
  if (!str)
    return NULL;
  
  len = strlen (str);
  ext = getext (str);
  
  if (ext)
    len -= strlen (ext) + 1;
  
  str[len] = 0;
  
  return str;
}

/* malloc and fill a pointer with a string */
char *
mystrdup (const char *string)
{
  char *ret;

  if (!string)
    return NULL;

  ret = malloc (strlen (string)+1);

  if (ret)
    strcpy (ret, string);
  return ret;
}

/* return the string of the time given as argument (in seconds) */
char *
time2s (int timeins /* Time in seconds */)
{
  char *ret;
  int sec, min, hour;

  hour =  timeins / 3600;
  min  = (timeins - hour*3600) / 60;
  sec  =  timeins - hour*3600 - min*60;

  ret = malloc ((hour ? 1 : 0) * 3 + (min ? 1 : 0) * 3 + 2 + 1);
  if (ret != NULL)
  {
    if (hour)
      sprintf (ret, "%02i:", hour);
    if (min)
    {
      if (hour)
      {
        sprintf (ret+3, "%02i:", min);
        sprintf (ret+6, "%02i", sec);
      }
      else
      {
        sprintf (ret, "%02i:", min);
        sprintf (ret+3, "%02i", sec);
      }
    }
    else
      sprintf (ret, "%02i", sec);
  }
  return ret;
}

/**
 * \brief Gets the length in 'characters' of an integer
 * \param n     an integer
 * \param base  the base to calculate the length (generally 10)
 * \return the length in characters of \p n
 * 
 * This function gets the length of the given number in its string
 * representation. I.e., 321 has a length of 3, and -554 has a length of 4.
 * 
 * \note Note that the \p base parameter cannot be 0. If it is 0, the function
 *       will return 0. It has no sense, but a base of 0 has not either.
 * \note The calculation with a base 1 is not improved; but what the hell is
 *       your fuckin' brain doing to let you call a function to get the length
 *       of a base 1 number?
 */
size_t intlen (long int n, unsigned int base)
{
  size_t len = 0;
  
  if (base <= 0)
    return 0;
  
  if (n < 0)
  {
    len++;
    n = -n;
  }
  
  do
  {
    n /= base;
    len++;
  } while (n > 0);
  
  return len;
}

size_t uintlen (unsigned long int n, unsigned int base)
{
  size_t len = 0;
  
  if (base <= 0)
    return 0;
  
  do
  {
    n /= base;
    len++;
  } while (n > 0);
  
  return len;
}

/* scandir doesn't exist in the mingw32 (windows) so make it (simplified) */
int
myscandir (const char *dir, struct dirent ***namelist,
           int (*select) (const struct dirent *))
{
  DIR *d;
  struct dirent *entry;
  register int i = 0;
  size_t entrysize;

  if ((d = opendir (dir)) == NULL)
    return -1;

  *namelist = NULL;
  while ((entry = readdir (d)) != NULL)
  {
    if (select == NULL || (select != NULL && (*select) (entry)))
    {
      *namelist = realloc (*namelist, (i + 1) * sizeof (**namelist));
      
      if (*namelist == NULL)
        return -1;
      
      entrysize = sizeof (struct dirent) - sizeof (entry->d_name) +
                  strlen (entry->d_name) + 1;
      
      (*namelist)[i] = (struct dirent *) malloc (entrysize);
      if ((*namelist)[i] == NULL)
        return -1;
      
      memcpy ((*namelist)[i], entry, entrysize);
      i++;
    }
  }
  if (closedir (d))
    return -1;
  if (i == 0)
    return -1;

  return i;
}


#if (DEBUG)
void internal_debugmsg (const char* fmt, ...)
{
  va_list args;
  
  fprintf (stdout, "DBG: ");
  va_start (args, fmt);
  vfprintf (stdout, fmt, args);
  va_end (args);
}
#endif /* (DEBUG) */
